#!/usr/bin/env python
# -*- coding: utf-8 -*-
from datetime import timedelta
from mptt.models import TreeForeignKey

from django.db import IntegrityError
from django.test import TestCase
from django.db import models
from django.core.exceptions import ValidationError
from django.contrib.sites.models import Site
from django.contrib.auth import get_user_model
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from opps.channels.models import Channel


class ChannelFields(TestCase):

    def test_name(self):
        field = Channel._meta.get_field_by_name('name')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 60)

    def test_long_sland(self):
        field = Channel._meta.get_field_by_name('long_slug')[0]
        self.assertEqual(field.__class__, models.SlugField)
        self.assertEqual(field.max_length, 250)
        self.assertTrue(field.db_index)

    def test_layout(self):
        field = Channel._meta.get_field_by_name('layout')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 250)
        self.assertTrue(field.db_index)
        self.assertEqual(field.default, u"default")

    def test_description(self):
        field = Channel._meta.get_field_by_name('description')[0]
        self.assertEqual(field.__class__, models.CharField)
        self.assertEqual(field.max_length, 255)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)

    def test_show_in_menu(self):
        field = Channel._meta.get_field_by_name('show_in_menu')[0]
        self.assertEqual(field.__class__, models.BooleanField)
        self.assertFalse(field.default)

    def test_include_in_main_rss(self):
        field = Channel._meta.get_field_by_name('include_in_main_rss')[0]
        self.assertEqual(field.__class__, models.BooleanField)
        self.assertTrue(field.default)

    def test_homepage(self):
        field = Channel._meta.get_field_by_name('homepage')[0]
        self.assertEqual(field.__class__, models.BooleanField)
        self.assertFalse(field.default)
        self.assertEqual(field.help_text,
                         u'Check only if this channel is the homepage.'
                         u' Should have only one homepage per site')

    def test_group(self):
        field = Channel._meta.get_field_by_name('group')[0]
        self.assertEqual(field.__class__, models.BooleanField)
        self.assertFalse(field.default)

    def test_order(self):
        field = Channel._meta.get_field_by_name('order')[0]
        self.assertEqual(field.__class__, models.IntegerField)
        self.assertEqual(field.default, 0)

    def test_parent(self):
        field = Channel._meta.get_field_by_name('parent')[0]
        self.assertEqual(field.__class__, TreeForeignKey)
        self.assertTrue(field.null)
        self.assertTrue(field.blank)


class ChannelModelTest(TestCase):

    def setUp(self):
        User = get_user_model()
        self.user = User.objects.create(username=u'test', password='test')
        self.site = Site.objects.filter(name=u'example.com').get()
        self.parent = Channel.objects.create(name=u'Home', slug=u'home',
                                             description=u'home page',
                                             site=self.site, user=self.user)

    def test_check_create_home(self):
        """
        Check exist channel home, create on setup class
        """
        home = Channel.objects.filter(slug=u'home').get()
        self.assertTrue(home)
        self.assertEqual(home, self.parent)

    def test_not_is_published(self):
        """
        is_published false on home channel
        """
        home = Channel.objects.filter(slug=u'home').get()
        self.assertFalse(home.is_published())
        self.assertEqual(home.is_published(), home.published)

    def test_is_published(self):
        """
        is_published true on home channel
        """
        self.parent.published = True
        self.parent.date_available = timezone.now() - timedelta(hours=1)
        self.parent.save()

        home = Channel.objects.filter(slug=u'home').get()
        self.assertTrue(home.is_published())
        self.assertEqual(home.is_published(), home.published)

    def test_create_sub_channel_home(self):
        channel = Channel.objects.create(name=u'Sub Home', slug=u'sub-home',
                                         description=u'sub home page',
                                         site=self.site, parent=self.parent,
                                         user=self.user)

        self.assertTrue(channel)
        self.assertEqual(channel.parent, self.parent)

    def test_not_is_published_sub_channel(self):
        """
        is_published false on home sub channel
        """
        subchannel = Channel.objects.create(name=u'Sub Home', slug=u'sub-home',
                                            description=u'sub home page',
                                            site=self.site, user=self.user,
                                            parent=self.parent)

        subhome = Channel.objects.filter(slug=u'sub-home').get()
        self.assertFalse(subhome.is_published())
        self.assertEqual(subhome.is_published(), subhome.published)
        self.assertEqual(subhome.is_published(), subchannel.is_published())

    def test_is_published_sub_channel(self):
        """
        is_published true on home sub channel
        """
        subchannel = Channel.objects.create(name=u'Sub Home', slug=u'sub-home',
                                            description=u'sub home page',
                                            site=self.site, user=self.user,
                                            parent=self.parent)
        subchannel.published = True
        subchannel.date_available = timezone.now() - timedelta(hours=1)
        subchannel.save()

        subhome = Channel.objects.filter(slug=u'sub-home').get()
        self.assertTrue(subhome.is_published())
        self.assertEqual(subhome.is_published(), subhome.published)
        self.assertEqual(subhome.is_published(), subhome.is_published())

    def test_duplicate_channel_name(self):
        """
        create 2 channel with same name
        """
        Channel.objects.create(name=u'Sub Home', slug=u'sub-home',
                               description=u'sub home page', site=self.site,
                               parent=self.parent, user=self.user)

        self.assertRaises(IntegrityError, Channel.objects.create,
                          name=u'Sub Home', slug=u'sub-home',
                          description=u'sub home page', site=self.site,
                          parent=self.parent)

    def test_channel_is_homepage(self):
        """
        check channel is home page
        """
        self.parent.homepage = True
        self.parent.published = True
        self.parent.save()

        channel = Channel.objects.get_homepage(site=self.parent.site)
        self.assertTrue(channel)
        self.assertEqual(channel.slug, self.parent.slug)

    def test_not_set_homepage(self):
        """
        not set channel home page, return none
        """

        channel = Channel.objects.get_homepage(site=self.parent.site)
        self.assertEqual(None, channel)
        self.assertFalse(channel)

    def test_get_thumb(self):
        self.assertIsNone(self.parent.get_thumb())

    def test_search_category(self):
        self.assertEqual(_(u'Channel'), Channel().search_category)

    def test_title(self):
        self.assertEqual(self.parent.title, self.parent.name)

    def test_root(self):
        self.assertEqual(self.parent, self.parent.root)
        subchannel = Channel.objects.create(name=u'Sub Home', slug=u'sub-home',
                                            description=u'sub home page',
                                            site=self.site, user=self.user,
                                            parent=self.parent)
        self.assertEqual(self.parent, subchannel.root)

    def test_clean_slug_exist_in_domain(self):
        invalid = Channel(name=u'Home', slug=u'home', description=u'home page',
                          site=self.site, user=self.user)
        self.assertRaises(ValidationError, invalid.full_clean)

    def test_clean_slug_exist_homepage(self):
        Channel.objects.create(name=u'homepage', slug=u'homepage',
                               homepage=True, site=self.site,
                               user=self.user, published=True)
        invalid = Channel.objects.create(name=u'invalid', slug=u'invalid',
                                         homepage=True, site=self.site,
                                         user=self.user)
        self.assertRaises(ValidationError, invalid.full_clean)

    def test_clean(self):
        valid = Channel.objects.create(name=u'homepage', slug=u'homepage',
                                       homepage=True, site=self.site,
                                       user=self.user, published=True)
        valid.full_clean()

    def test_long_slug(self):
        channel1 = Channel.objects.create(name=u'Channel 1', slug=u'channel1',
                                          parent=self.parent,
                                          description=u'channel 1',
                                          site=self.site, user=self.user)
        channel2 = Channel.objects.create(name=u'Channel 2', slug=u'channel2',
                                          parent=channel1,
                                          description=u'channel 2',
                                          site=self.site, user=self.user)
        self.assertEqual(self.parent.long_slug, 'home')
        self.assertEqual(channel1.long_slug, 'home/channel1')
        self.assertEqual(channel2.long_slug, 'home/channel1/channel2')

    def test_unicode(self):
        channel1 = Channel.objects.create(name=u'Channel 1', slug=u'channel1',
                                          parent=self.parent,
                                          description=u'channel 1',
                                          site=self.site, user=self.user)
        channel2 = Channel.objects.create(name=u'Channel 2', slug=u'channel2',
                                          parent=channel1,
                                          description=u'channel 2',
                                          site=self.site, user=self.user)
        self.assertEqual(self.parent.__unicode__(), '/home/')
        self.assertEqual(channel1.__unicode__(), '/home/channel1/')
        self.assertEqual(channel2.__unicode__(), '/home/channel1/channel2/')

    def test_get_absolute_url(self):
        self.test_unicode()



#!/usr/bin/env python
# -*- coding: utf-8 -*
import json
from django import forms
from django.template.loader import render_to_string

from .models import Field, FieldOption
from opps.core.widgets import CONFIG


class JSONField(forms.TextInput):
    model = Field

    def render(self, name, value, attrs=None):
        elements = []
        try:
            values = json.loads(value)
        except TypeError:
            values = {}

        # value sometimes come as unicode and we need to treat it
        if type(values) == unicode:
            values = json.loads(values)

        objs = self.model.objects.filter(
            application__contains=self.attrs.get('_model', None))

        for obj in objs:
            o = {}
            o['name'] = obj.name
            o['slug'] = obj.slug

            element_attr = {}
            element_attr['name'] = obj.name
            element_attr['slug'] = obj.slug
            """
            element_attr['value'] = '1'
            element_attr['obj_value'] = values.get(obj.slug, '')
            """
            element_attr['obj_value'] = values.get(obj.slug, '')

            if obj.type in ["checkbox", "radio"]:
                fo = FieldOption.objects.filter(field=obj)
                if obj.type == "checkbox":
                    obj_value = []
                    for i in fo:
                        key = "{0}_{1}".format(obj.slug, i.option.slug)
                        obj_value.append(values.get(key, ''))
                    element_attr['list'] = zip(fo, obj_value)
                    del element_attr['obj_value']
                else:
                    element_attr['list'] = fo

            o['element'] = render_to_string(
                "admin/opps/fields/json_{0}.html".format(obj.type),
                dictionary=element_attr
            )
            elements.append(o)

        # OPPS Editor params
        # This is ugly as hell but for now it was the only way of getting this
        # working DRY
        js = CONFIG.get('js')[0]
        # must pass an string with commas on plugins
        plugins = ''
        for item in CONFIG.get('plugins'):
            line = ','.join(item.split())
            line += ','
            plugins += line
        language = CONFIG.get('language')
        theme = CONFIG.get('theme', 'modern')
        file_browser_callback = CONFIG.get('file_browser_callback')

        return render_to_string(
            "admin/opps/fields/json.html",
            {"elements": elements,
             "name": name,
             "value": value,
             "js": js,
             "theme": theme,
             "plugins": plugins,
             "language": language,
             "file_browser_callback": file_browser_callback,
             })



# -*- coding: utf-8 -*-

from django.utils.translation import ugettext_lazy as _

from appconf import AppConf


trans_app_label = _('Core')


class OppsCoreConf(AppConf):
    DEFAULT_URLS = ('127.0.0.1', 'localhost',)
    SHORT = 'googl'
    SHORT_URL = 'googl.short.GooglUrlShort'
    CHANNEL_CONF = {}
    CHANNEL_LAYOUT = (('default', _('Default')),)
    VIEWS_LIMIT = None
    PAGINATE_BY = 10
    PAGINATE_SUFFIX = u''
    PAGINATE_NOT_APP = []
    CHECK_MOBILE = False
    DOMAIN_MOBILE = u''
    PROTOCOL_MOBILE = u'http'
    ADMIN_RULES = {}
    RELATED_POSTS_PLACEHOLDER = "---related---"
    CACHE_PREFIX = 'opps'
    CACHE_EXPIRE = 300
    CACHE_EXPIRE_LIST = 300
    CACHE_EXPIRE_DETAIL = 300
    RSS_LINK_TEMPLATE = '<a href="{}" class="ir ico ico-rss">RSS</a>'
    LIST_MODELS = ('Post',)
    RECOMMENDATION_RANGE_DAYS = 180
    SMART_SLUG_ENABLED = True
    MENU = True
    MENU_ONLY_WITH_PUBLISHED_CONTAINERS = False
    MIRROR_CHANNEL = False
    CONTAINERS_BLACKLIST = ['Entry']
    CONTAINERS_SITE_ID = None

    # default settings for tinymce
    EDITOR = {
        'editor': 'tinymce',
        'height': 400,
        'js': ('/static/tinymce/tinymce.min.js',),
        "theme": "modern",
        "plugins": [
            """advlist autolink lists link image charmap print preview hr
        anchor pagebreak """,
            "searchreplace wordcount visualblocks visualchars code fullscreen",
            """insertdatetime media nonbreaking save table contextmenu
        directionality""",
            "template paste textcolor opps"
        ],
        "toolbar1": """insertfile undo redo | styleselect | bold italic |
                alignleft aligncenter alignright alignjustify |
                bullist numlist outdent indent | link image media |
                print preview  | forecolor backcolor | opps""",
        "image_advtab": True,
        "templates": [
            {"title": 'Related', "content": RELATED_POSTS_PLACEHOLDER},
        ],
        "file_browser_callback": 'CustomFileBrowser',
    }

    class Meta:
        prefix = 'opps'


class GrapelliConf(AppConf):

    ADMIN_TITLE = "Opps CMS Admin"
    INDEX_DASHBOARD = 'opps.contrib.admin.dashboard.CustomIndexDashboard'

    class Meta:
        prefix = 'GRAPPELLI'


class AdminConf(AppConf):

    SHORTCUTS = [
        {
            'shortcuts': [
                {
                    'url_name': 'admin:articles_post_add',
                    'title': '+ Notícia',
                    'class': 'file3',
                    'help': 'Clique para adicionar uma nova notícia'
                },
                {
                    'url_name': 'admin:articles_post_changelist',
                    'title': 'Notícias',
                    'count': 'opps.contrib.admin.shortcuts.count_posts',
                    'class': 'file2',
                    'help': 'Clique para visualisar todas as notícias'
                },
                {
                    'url_name': 'admin:images_image_add',
                    'title': '+ Imagem',
                    'class': 'picture',
                    'help': 'Clique para adicionar uma nova imagem'
                },
                {
                    'url_name': 'admin:articles_album_changelist',
                    'title': 'Álbum',
                    'count': 'opps.contrib.admin.shortcuts.count_albums',
                    'class': 'camera',
                    'help': 'Clique para visualisar todos os álbuns'
                },
                {
                    'url': '/',
                    'open_new_window': True,
                    'help': 'Clique para visualizar a home page do site'
                },
            ]
        }
    ]

    SHORTCUTS_SETTINGS = {
        'hide_app_list': True,
        'open_new_window': False,
    }

    SHORTCUTS_CLASS_MAPPINGS_EXTRA = [
        ('blogs_blogpost', 'blog')
    ]

    class Meta:
        prefix = 'ADMIN'


class StaticSiteMapsConf(AppConf):
    ROOT_SITEMAP = 'opps.sitemaps.feed.sitemaps'

    class Meta:
        prefix = 'staticsitemaps'


class HaystackConf(AppConf):
    CONNECTIONS = {
        'default': {
            'ENGINE': 'haystack.backends.simple_backend.SimpleEngine',
        }
    }

    class Meta:
        prefix = 'haystack'


class ThumborConf(AppConf):
    SERVER = 'http://localhost:8888'
    MEDIA_URL = 'http://localhost:8000/media'
    SECURITY_KEY = ''
    ARGUMENTS = {}
    ENABLED = False

    class Meta:
        prefix = 'thumbor'


class DjangoConf(AppConf):
    CACHES = {'default': {
        'BACKEND': 'django.core.cache.backends.dummy.DummyCache'}}



import copy
import inspect
from django import template
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.utils.importlib import import_module

register = template.Library()


def make_url(shortcut, request):

    permission = shortcut.get('app_permission')
    if permission and \
            isinstance(permission, (list, tuple)) and len(permission) >= 2:
        shortcut['can_add'] = request.user.has_perm(
            '{0}.add_{1}'.format(*permission)
        )
        shortcut['can_change'] = request.user.has_perm(
            '{0}.change_{1}'.format(*permission)
        )

    if not shortcut.get('url'):
        try:
            url_name = shortcut['url_name']
        except KeyError:
            raise ImproperlyConfigured(
                _('settings.ADMIN_SHORTCUTS is improperly configured. '
                  'Please supply either a "url" or a "url_name" '
                  'for each shortcut.')
            )
        current_app = shortcut.get('app_name')
        if isinstance(url_name, list):
            shortcut['url'] = reverse(
                url_name[0], args=url_name[1:], current_app=current_app
            )
        else:
            shortcut['url'] = reverse(
                url_name, current_app=current_app)

        shortcut['url'] += shortcut.get('url_extra', '')

    if not shortcut.get('class'):
        shortcut['class'] = get_shortcut_class(
            shortcut.get('url_name', shortcut['url']))

    if shortcut.get('count'):
        shortcut['count'] = eval_func(shortcut['count'], request)

    if shortcut.get('count_new'):
        shortcut['count_new'] = eval_func(
            shortcut['count_new'], request)


def add_permission(parent, child):
    if child.get('can_change', None):
        parent['can_change'] = True


@register.inclusion_tag('admin_shortcuts/base.html', takes_context=True)
def admin_shortcuts(context):
    if 'ADMIN_SHORTCUTS' in context:
        admin_shortcuts = copy.deepcopy(context['ADMIN_SHORTCUTS'])
    else:
        admin_shortcuts = copy.deepcopy(
            getattr(settings, 'ADMIN_SHORTCUTS', []))
    admin_shortcuts_settings = copy.deepcopy(
        getattr(settings, 'ADMIN_SHORTCUTS_SETTINGS', {}))
    request = context.get('request', None)
    if not admin_shortcuts:
        return ''

    for group in admin_shortcuts:
        if not group.get('shortcuts'):
            raise ImproperlyConfigured(
                'settings.ADMIN_SHORTCUTS is improperly configured.')

        for shortcut in group.get('shortcuts', []):
            make_url(shortcut, request)
            add_permission(group, shortcut)

            for child in shortcut.get('children', []):
                make_url(child, request)
                add_permission(shortcut, child)
                # TODO: make it recursive for submenus

    return {
        'admin_shortcuts': admin_shortcuts,
        'settings': admin_shortcuts_settings,
        'context': context
    }


@register.inclusion_tag('admin_shortcuts/style.css')
def admin_shortcuts_css():
    return {
        'classes': [value for key, value in CLASS_MAPPINGS],
    }


@register.inclusion_tag('admin_shortcuts/js.html')
def admin_shortcuts_js():
    admin_shortcuts_settings = getattr(
        settings, 'ADMIN_SHORTCUTS_SETTINGS', None)
    return {
        'settings': admin_shortcuts_settings,
    }


def eval_func(func_path, request):
    try:
        module_str = '.'.join(func_path.split('.')[:-1])
        func_str = func_path.split('.')[-1:][0]
        module = import_module(module_str)
        result = getattr(module, func_str)
        if callable(result):
            args, varargs, keywords, defaults = inspect.getargspec(result)
            if 'request' in args:
                result = result(request)
            else:
                result = result()
        return result
    except:
        return func_path


@register.simple_tag
def admin_static_url():
    """
    If set, returns the string contained in the setting
    ADMIN_MEDIA_PREFIX, otherwise returns STATIC_URL + 'admin/'.
    """
    return getattr(settings, 'ADMIN_MEDIA_PREFIX', None) \
        or ''.join([settings.STATIC_URL, 'admin/'])


def get_shortcut_class(url):
    if url == '/':
        return 'home'
    for key, value in CLASS_MAPPINGS:
        if key is not None and key in url:
            return value
    return 'config'  # default icon


XTRA = getattr(settings, 'ADMIN_SHORTCUTS_CLASS_MAPPINGS_EXTRA', [])

CLASS_MAPPINGS = XTRA + getattr(settings, 'ADMIN_SHORTCUTS_CLASS_MAPPINGS', [
    ['cms_page', 'file2'],
    ['product', 'basket'],
    ['order', 'cash'],
    ['category', 'archive'],
    ['user', 'user'],
    ['account', 'user'],
    ['address', 'letter'],
    ['folder', 'folder'],
    ['gallery', 'picture'],
    ['blog', 'blog'],
    ['event', 'date'],
    ['mail', 'openmail'],
    ['message', 'openmail'],
    ['contact', 'openmail'],
    ['location', 'pin'],
    ['store', 'pin'],
    ['delivery', 'delivery2'],
    ['shipping', 'delivery2'],
    ['add', 'plus'],
    ['change', 'pencil'],
    ['home', 'home'],

    # extra classes
    [None, 'archive'],
    [None, 'back'],
    [None, 'camera'],
    [None, 'card'],
    [None, 'cd'],
    [None, 'certificate'],
    [None, 'clock'],
    [None, 'cloud1'],
    [None, 'cloud2'],
    [None, 'cloud3'],
    [None, 'cloud4'],
    [None, 'config'],
    [None, 'config2'],
    [None, 'date'],
    [None, 'delivery1'],
    [None, 'diskette'],
    [None, 'file1'],
    [None, 'file3'],
    [None, 'file4'],
    [None, 'film'],
    [None, 'flag'],
    [None, 'gamepad'],
    [None, 'garbage'],
    [None, 'gift'],
    [None, 'help'],
    [None, 'key'],
    [None, 'less'],
    [None, 'letters'],
    [None, 'light'],
    [None, 'lock'],
    [None, 'love'],
    [None, 'mail'],
    [None, 'monitor'],
    [None, 'music'],
    [None, 'note'],
    [None, 'notepad'],
    [None, 'ok'],
    [None, 'package'],
    [None, 'phone'],
    [None, 'pin'],
    [None, 'print'],
    [None, 'sound'],
    [None, 'suitcase'],
    [None, 'tag'],
    [None, 'ticket'],
    [None, 'tool'],
    [None, 'unlock'],
    [None, 'wallet'],
    [None, 'warning'],
    [None, 'way'],
    [None, 'zoom'],
])



# coding: utf-8
import csv

from django.http import HttpResponse
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone


def export_to_csv(modeladmin, request, queryset):
    """Exporting queryset results and filter into CSV"""
    # limit only staff and super_user accounts
    if request.user.is_staff or request.user.is_superuser:
        if queryset.count() > 0:
            # generate response and filename
            response = HttpResponse(mimetype="text/csv")
            today = timezone.now().strftime("%Y-%M-%d_%H:%M:%S")
            filename = "{0}-{1}.csv".format(queryset.model, today)
            response["Content-Disposition"] = ('attachment; filename="%s"' %
                                               filename)
            writer = csv.writer(response)

            # Get column name
            columns = [field.name for field in queryset[0]._meta.fields]
            writer.writerow(columns)

            # Write data
            for obj in queryset:
                fields = map(lambda x: _generate_value(obj, x), columns)
                writer.writerow(fields)

            return response

export_to_csv.short_description = _(u"Export results in CSV")


def _generate_value(obj, column):
    """Get fields value and convert to ASCIC for string type"""
    row = getattr(obj, column)
    if isinstance(row, basestring):
        row = row.encode('ascii', 'ignore')
    return row



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.test import TestCase
from django.contrib.sites.models import Site
from django.contrib.auth import get_user_model

from opps.channels.models import Channel
from opps.core.widgets import OppsEditor

from opps.articles.models import Post, Album
from opps.articles.forms import PostAdminForm, AlbumAdminForm


class PostAdminFormTest(TestCase):

    def setUp(self):
        User = get_user_model()
        self.user = User.objects.create(username=u'test', password='test')
        self.site = Site.objects.filter(name=u'example.com').get()
        self.channel = Channel.objects.create(name=u'Home', slug=u'home',
                                              description=u'home page',
                                              site=self.site, user=self.user)
        self.post = Post.objects.create(title=u'test', user=self.user,
                                        site=self.site, channel=self.channel)

    def test_init(self):
        """
        Test successful init without data
        """
        form = PostAdminForm(instance=self.post)
        self.assertTrue(isinstance(form.instance, Post))
        self.assertEqual(form.instance.pk, self.post.pk)

    def test_default_multiupload_link(self):
        """
        Test default value field multiupload link
        """
        form = PostAdminForm(instance=self.post)
        self.assertEqual(form.multiupload_link, '/fileupload/image/')

    def test_editor_widgets(self):
        """
        Test auto set field widget Editor
        """
        form = PostAdminForm(instance=self.post)
        self.assertTrue(isinstance(form.fields['content'].widget,
                                   OppsEditor))


class AlbumAdminFormTest(TestCase):

    def setUp(self):
        User = get_user_model()
        self.user = User.objects.create(username=u'test', password='test')
        self.site = Site.objects.filter(name=u'example.com').get()
        self.channel = Channel.objects.create(name=u'Home', slug=u'home',
                                              description=u'home page',
                                              site=self.site, user=self.user)
        self.album = Album.objects.create(title=u'test', user=self.user,
                                          site=self.site, channel=self.channel)

    def test_init(self):
        """
        Test successful init without data
        """
        form = AlbumAdminForm(instance=self.album)
        self.assertTrue(isinstance(form.instance, Album))
        self.assertEqual(form.instance.pk, self.album.pk)

    def test_default_multiupload_link(self):
        """
        Test default value field multiupload link
        """
        form = AlbumAdminForm(instance=self.album)
        self.assertEqual(form.multiupload_link, '/fileupload/image/')

    def test_editor_widgets(self):
        """
        Test auto set field widget Editor
        """
        form = AlbumAdminForm(instance=self.album)
        self.assertTrue(isinstance(form.fields['headline'].widget,
                                   OppsEditor))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.views.generic.detail import DetailView as DjangoDetailView
from django.contrib.sites.models import Site, get_current_site
from django.http import HttpResponseRedirect
from django.utils import timezone
from django.conf import settings
from opps.views.generic.base import View


class DetailView(View, DjangoDetailView):

    def get_template_list(self, domain_folder="containers"):
        child_class = self.object.child_class.lower()

        templates = ['{0}/{1}/{2}/detail.html'.format(
            domain_folder, self.long_slug, self.slug)]

        all_long_slug = [i.long_slug for i in self.channel.get_ancestors()]
        all_long_slug.append(self.long_slug)
        for l in all_long_slug[::-1]:
            templates.append('{0}/{1}/{2}_detail.html'.format(
                domain_folder, l, child_class))
        templates.append('{0}/{1}_detail.html'.format(domain_folder,
                                                      child_class))

        for l in all_long_slug[::-1]:
            templates.append('{0}/{1}/detail.html'.format(domain_folder, l))
        templates.append('{0}/detail.html'.format(domain_folder))

        return templates

    def get_template_names(self):
        if self.template_name:
            return [self.template_name]

        domain_folder = self.get_template_folder()
        template_list = self.get_template_list(domain_folder)

        if domain_folder != "containers":
            template_list.extend(self.get_template_list())

        return template_list

    def render_to_response(self, context):
        if self.get_object().child_class == 'Link':
            return HttpResponseRedirect(self.get_object().url)
        return super(DetailView, self).render_to_response(context)

    def get_queryset(self):
        self.fallback = getattr(settings, 'OPPS_MULTISITE_FALLBACK', False)
        self.site = get_current_site(self.request)
        self.site_master = Site.objects.order_by('id')[0]
        self.slug = self.kwargs.get('slug')
        self.long_slug = self.get_long_slug()
        if not self.long_slug:
            return None

        self.set_channel_rules()

        filters = {}
        filters['site_domain'] = self.site.domain
        filters['channel_long_slug'] = self.long_slug
        filters['slug'] = self.slug

        preview_enabled = self.request.user and self.request.user.is_staff

        if not preview_enabled:
            filters['date_available__lte'] = timezone.now()
            filters['published'] = True

        queryset = super(DetailView, self).get_queryset()

        qs = queryset.filter(**filters)._clone()
        if not qs and self.fallback and self.site_master != self.site:
            filters['site_domain'] = self.site_master.domain
            qs = queryset.filter(**filters)._clone()
        return qs




from HTMLParser import HTMLParser
from django import template

register = template.Library()
parser = HTMLParser()

import xml.parsers.expat


def unescape(s):
    want_unicode = False
    if isinstance(s, unicode):
        s = s.encode("utf-8")
        want_unicode = True

    # the rest of this assumes that `s` is UTF-8
    list = []

    # create and initialize a parser object
    p = xml.parsers.expat.ParserCreate("utf-8")
    p.buffer_text = True
    p.returns_unicode = want_unicode
    p.CharacterDataHandler = list.append

    # parse the data wrapped in a dummy element
    # (needed so the "document" is well-formed)
    p.Parse("<e>", 0)
    p.Parse(s, 0)
    p.Parse("</e>", 1)

    # join the extracted strings and return
    es = ""
    if want_unicode:
        es = u""
    return es.join(list)


@register.simple_tag
def htmlentitydecode(s):
    try:
        unescaped = unescape(parser.unescape(s))
        return unescaped
    except:
        unescaped = parser.unescape(s)
        return unescaped.encode('utf-8')

register.filter('htmlentitydecode', htmlentitydecode)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.contrib.redirects.models import Redirect


def redirect_generate(sender, instance, created, **kwargs):
    obj, create = Redirect.objects.get_or_create(
        old_path=instance.get_absolute_url(),
        site=instance.site,
        new_path=instance.url)
    obj.save()



# coding: utf-8

import xmltodict
from django.http import HttpResponse
from django.views.generic import TemplateView
from django.views.generic.detail import BaseDetailView
from django.views.generic.detail import SingleObjectTemplateResponseMixin


def cdata(data):
    if not data:
        return u""
    return u"<!CDATA[ {0} ]]>".format(data)


def response_mimetype(request):
    if "application/xml" in request.META['HTTP_ACCEPT']:
        return "application/xml"
    return "text/plain"


class XMLResponse(HttpResponse):
    """XML response class.
    takes a python dict and returns a XML string
    obj = {
        'results':
            {
                '@val': '1',
                'name': '3',
                '#text': 'sometext',
                'items': {'item': ['1', '2', '3']}
            }
    }
    in to --->
    <?xml version="1.0" encoding="utf-8"?>
    <results val="1">
        <name>3</name>
        <items>
           <item>1</item>
           <item>2</item>
           <item>3</item>
        </items>
        sometext
    </results>
    """
    def __init__(self, obj='', mimetype="application/xml", *args, **kwargs):
        content = xmltodict.unparse(obj)
        super(XMLResponse, self).__init__(content, mimetype, *args, **kwargs)


class XMLResponseMixin(object):
    """
    A mixin that can be used to render a XML response.
    """
    def render_to_xml_response(self, context, **response_kwargs):
        """
        Returns a XML response, transforming 'context' to make the payload.
        """
        return HttpResponse(
            self.convert_context_to_xml(context),
            content_type='application/xml',
            **response_kwargs
        )

        # response = XMLResponse(
        #     self.convert_context_to_xml(context),
        #     {},
        #     response_mimetype(self.request)
        # )
        # response['Content-Disposition'] = 'inline; filename=files.xml'

    def convert_context_to_xml(self, context):
        "Convert the context dictionary into a XML object"
        # Note: This is *EXTREMELY* naive; in reality, you'll need
        # to do much more complex handling to ensure that arbitrary
        # objects -- such as Django model instances or querysets
        # -- can be serialized as XML.
        return xmltodict.unparse(context)


class XMLView(XMLResponseMixin, TemplateView):
    def render_to_response(self, context, **response_kwargs):
        return self.render_to_xml_response(context, **response_kwargs)


class XMLDetailView(XMLResponseMixin, BaseDetailView):
    def render_to_response(self, context, **response_kwargs):
        return self.render_to_xml_response(context, **response_kwargs)


class HybridDetailView(XMLResponseMixin,
                       SingleObjectTemplateResponseMixin,
                       BaseDetailView):
    def render_to_response(self, context):
        # Look for a 'format=xml' GET argument
        if self.request.GET.get('format') == 'xml':
            return self.render_to_xml_response(context)
        else:
            return super(HybridDetailView, self).render_to_response(context)



from django.test import TestCase
from django.test.client import RequestFactory

from opps.contrib.mobile import template
from opps.contrib.mobile.middleware import (
    MobileDetectionMiddleware, MobileRedirectMiddleware
)


class TestMobileTemplatesDir(TestCase):

    def setUp(self):
        self.detection_middleware = MobileDetectionMiddleware()
        self.factory = RequestFactory()
        self.template_loader = template.Loader()

    def test_useragent_based_templatedirs(self):
        # Override the TEMPLATE_LOADERS and MIDDLEWARE_CLASSES settings
        # to use the middlewares in ``opps.contrib.mobile.middleware``
        # and the ``opps.contrib.mobile.template.Loader``
        MIDDLEWARE_CLASSES = (
            'django.middleware.common.CommonMiddleware',
            'opps.contrib.mobile.middleware.MobileDetectionMiddleware',
            'opps.contrib.mobile.middleware.MobileRedirectMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
            'django.contrib.redirects.middleware.RedirectFallbackMiddleware',
        )
        TEMPLATE_LOADERS = (
            'opps.contrib.mobile.template.Loader',
            'django.template.loaders.app_directories.Loader',
        )
        TEMPLATE_DIRS_MOBILE = ('mobile-templates',) 
        TEMPLATE_DIRS_WEB = ('web-templates',)

        custom_settings = self.settings(
            MIDDLEWARE_CLASSES=MIDDLEWARE_CLASSES,
            TEMPLATE_LOADERS=TEMPLATE_LOADERS,
            TEMPLATE_DIRS_MOBILE=TEMPLATE_DIRS_MOBILE,
            TEMPLATE_DIRS_WEB=TEMPLATE_DIRS_WEB,
            OPPS_CHECK_MOBILE=True,
            OPPS_DOMAIN_MOBILE = 'm.testserver'
        )
        with custom_settings:
            mobile_request = self.factory.get('/', HTTP_USER_AGENT='mobi')
            desktop_request = self.factory.get('/', HTTP_USER_AGENT='Mozilla/5.0')

            self.detection_middleware.process_request(desktop_request)
            self.assertEqual(
                self.template_loader.get_template_sources('index.html').next(),
                self.template_loader.get_template_sources('index.html', TEMPLATE_DIRS_WEB).next()
            )

            self.detection_middleware.process_request(mobile_request)
            self.assertEqual(
                self.template_loader.get_template_sources('index.html').next(),
                self.template_loader.get_template_sources('index.html', TEMPLATE_DIRS_MOBILE).next()
            )



# -*- coding: utf-8 -*-
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from mptt.admin import MPTTModelAdmin

from .models import Channel
from .forms import ChannelAdminForm
from opps.core.admin import PublishableAdmin
from opps.core.admin import apply_opps_rules
from opps.core.utils import get_template_path

import json


@apply_opps_rules('channels')
class ChannelAdmin(PublishableAdmin, MPTTModelAdmin):
    prepopulated_fields = {"slug": ("name",)}
    list_display = ['name', 'show_channel_path', 'parent', 'site',
                    'date_available', 'homepage', 'order', 'show_in_menu',
                    'published']
    list_filter = ['date_available', 'published', 'site', 'homepage', 'parent',
                   'show_in_menu']
    search_fields = ['name', 'slug', 'long_slug', 'description']
    exclude = ('user', 'long_slug')
    raw_id_fields = ['parent', 'main_image']
    form = ChannelAdminForm

    fieldsets = (
        (_(u'Identification'), {
            'fields': ('site', 'parent', 'name', 'slug', 'layout', 'hat',
                       'description', 'main_image',
                       'order', ('show_in_menu', 'include_in_main_rss'),
                       'homepage', 'group', 'paginate_by')}),
        (_(u'Publication'), {
            'classes': ('extrapretty'),
            'fields': ('published', 'date_available')}),
    )

    def show_channel_path(self, obj):
        return unicode(obj)
    show_channel_path.short_description = _(u'Channel Path')

    def save_model(self, request, obj, form, change):
        long_slug = u"{0}".format(obj.slug)
        if obj.parent:
            long_slug = u"{0}/{1}".format(obj.parent.slug, obj.slug)
        obj.long_slug = long_slug

        super(ChannelAdmin, self).save_model(request, obj, form, change)

    def get_form(self, request, obj=None, **kwargs):
        form = super(ChannelAdmin, self).get_form(request, obj, **kwargs)

        channel_json = []

        def _get_template_path(_path):
            template = get_template_path(_path)
            with open(template) as f:
                _jsonData = f.read().replace('\n', '')
                return json.loads(_jsonData)

        def _get_json_channel(_obj):
            return _get_template_path(
                u'containers/{0}/channel.json'.format(_obj.long_slug))

        def _get_json_channel_recursivelly(_obj):
            channel_json = []
            try:
                channel_json = _get_json_channel(_obj)
            except:
                _is_root = _obj.is_root_node()
                if not _is_root:
                    channel_json = _get_json_channel_recursivelly(_obj.parent)
                elif _is_root:
                    try:
                        channel_json = _get_template_path(
                            u'containers/channel.json')
                    except:
                        pass
            finally:
                return channel_json

        channel_json = _get_json_channel_recursivelly(obj)

        if u'layout' in channel_json:
            layout_list = ['default'] + [l for l in channel_json['layout']]
            layout_choices = (
                (n, n.title()) for n in layout_list)

            form.base_fields['layout'].choices = layout_choices
        return form


admin.site.register(Channel, ChannelAdmin)



# -*- coding: utf-8 -*-
import logging

from django import template
from django.core.cache import cache
from django.contrib.sites.models import Site

register = template.Library()


@register.assignment_tag
def get_all_sites():
    """
    Takes the container object and get recommendations and filters by given
    child_class and limit the result by given slice.

    Sample usage::

        {% get_all_sites as context_var %}
    """

    return Site.objects.all()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from googl.short import GooglUrlShort
from django.conf import settings

from opps.channels.models import Channel

GENERATE_SHORT_URL = getattr(settings, 'GENERATE_SHORT_URL', False)


def shorturl_generate(sender, instance, created, **kwargs):
    if GENERATE_SHORT_URL:
        if not instance.short_url:
            instance.short_url = GooglUrlShort(
                instance.get_http_absolute_url()
            ).short()
            instance.save()


def delete_container(sender, instance, using, **kwargs):
    """
    Ensure objects are deleted from mother class Article
    when deleted from child class
    """
    try:
        instance.__class__.objects.filter(
            child_class=instance.child_class,
            slug=instance.slug,
            channel=instance.channel,
            site=instance.site
        ).delete()
    except (instance.__class__.DoesNotExist, Channel.DoesNotExist):
        # object not exists
        pass



# -*- coding: utf-8 -*-

from django.conf import settings
from django import forms

from .models import Channel


class ChannelAdminForm(forms.ModelForm):
    layout = forms.ChoiceField(choices=settings.OPPS_CHANNEL_LAYOUT)

    def __init__(self, *args, **kwargs):
        super(ChannelAdminForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        if instance and instance.pk:
            self.fields['slug'].widget.attrs['readonly'] = True

    class Meta:
        model = Channel



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from opps.core.models import Publishable
from django.utils.translation import ugettext_lazy as _


class SitePermission(Publishable):
    """
    Join user in site
    """
    class Meta:
        verbose_name = _(u'Site Permission')
        verbose_name_plural = _(u'Site Permissions')

    def __unicode__(self):
        return u'{0} em {1}'.format(self.user, self.site)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Tag'
        db.create_table(u'tags_tag', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('date_insert', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('date_update', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=150)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255, db_index=True)),
        ))
        db.send_create_signal(u'tags', ['Tag'])

        # Adding unique constraint on 'Tag', fields ['slug', 'name']
        db.create_unique(u'tags_tag', ['slug', 'name'])


    def backwards(self, orm):
        # Removing unique constraint on 'Tag', fields ['slug', 'name']
        db.delete_unique(u'tags_tag', ['slug', 'name'])

        # Deleting model 'Tag'
        db.delete_table(u'tags_tag')


    models = {
        u'tags.tag': {
            'Meta': {'unique_together': "(['slug', 'name'],)", 'object_name': 'Tag'},
            'date_insert': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_update': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255', 'db_index': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '150'})
        }
    }

    complete_apps = ['tags']


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding M2M table for field mirror_site on 'SitePermission'
        m2m_table_name = db.shorten_name(u'multisite_sitepermission_mirror_site')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('sitepermission', models.ForeignKey(orm[u'multisite.sitepermission'], null=False)),
            ('site', models.ForeignKey(orm[u'sites.site'], null=False))
        ))
        db.create_unique(m2m_table_name, ['sitepermission_id', 'site_id'])


    def backwards(self, orm):
        # Removing M2M table for field mirror_site on 'SitePermission'
        db.delete_table(db.shorten_name(u'multisite_sitepermission_mirror_site'))


    models = {
        u'accounts.customuser': {
            'Meta': {'object_name': 'CustomUser'},
            'accept_terms': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'birthday': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'blog_url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'complement': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'country': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'cpf': ('django.db.models.fields.CharField', [], {'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'district': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'unique': 'True', 'max_length': '75', 'db_index': 'True'}),
            'facebook_url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'gender': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True', 'blank': 'True'}),
            'google_url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'main_image': ('django.db.models.fields.files.FileField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'mobile_phone': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'number': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'rg': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'}),
            'street': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'twitter_url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'zipcode': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'})
        },
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'multisite.sitepermission': {
            'Meta': {'object_name': 'SitePermission'},
            'date_available': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'null': 'True', 'db_index': 'True'}),
            'date_insert': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_update': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mirror_site': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "u'multisite_sitepermission_mirror_site'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['sites.Site']"}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': u"orm['sites.Site']"}),
            'site_domain': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'site_iid': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True', 'max_length': '4', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['accounts.CustomUser']"})
        },
        u'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['multisite']


#!/usr/bin/env python
# -*- coding:utf-8 -*-
import sys
from setuptools import setup, find_packages

import opps


REQUIREMENTS = [i.strip() for i in open("requirements.txt").readlines()]
if sys.argv[1] == 'develop':
    REQUIREMENTS += [i.strip()
                     for i in open("requirements_dev.txt").readlines()]


dependency_links = [
    'http://github.com/avelino/django-googl/tarball/master#egg=django-googl',
    'http://github.com/opps/opps-piston/tarball/master#egg=opps-piston',
]

classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Web Environment',
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Framework :: Django",
    "Framework :: Opps",
    'Programming Language :: Python',
    "Programming Language :: Python :: 2.7",
    "Operating System :: OS Independent",
    "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    'Topic :: Software Development :: Libraries :: Application Frameworks',
    'Topic :: Software Development :: Libraries :: Python Modules']

try:
    long_description = open('README.rst').read()
except:
    long_description = opps.__description__

setup(name='opps',
      version=opps.__version__,
      description=opps.__description__,
      long_description=long_description,
      classifiers=classifiers,
      keywords='opps cms django apps magazines websites',
      author=opps.__author__,
      author_email=opps.__email__,
      url='http://oppsproject.org',
      download_url="https://github.com/opps/opps/tarball/master",
      license=opps.__license__,
      packages=find_packages(exclude=('doc', 'docs', 'example')),
      namespace_packages=['opps'],
      package_dir={'opps': 'opps'},
      install_requires=REQUIREMENTS,
      dependency_links=dependency_links,
      scripts=['opps/bin/opps-admin.py'],
      include_package_data=True,
      test_suite='runtests',
      zip_safe=False)



# -*- coding: utf-8 -*-

from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from opps.db.models.fields.jsonf import JSONFormField
from opps.fields.widgets import JSONField
from opps.fields.models import Field, FieldOption

from .models import Container, ContainerBoxContainers


class ContainerBoxContainersInlineForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(ContainerBoxContainersInlineForm,
              self).__init__(*args, **kwargs)
        self.fields['order'].widget.attrs['readonly'] = True

    class Meta:
        model = ContainerBoxContainers


class ContainerAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(ContainerAdminForm, self).__init__(*args, **kwargs)

        if not settings.OPPS_MIRROR_CHANNEL:
            self.fields['mirror_channel'].widget.attrs['disabled'] = True

        self.fields['json'] = JSONFormField(
            widget=JSONField(
                attrs={'_model': self._meta.model.__name__}),
            required=False)
        for field in Field.objects.filter(
                application__contains=self._meta.model.__name__):
            if field.type == 'checkbox':
                for fo in FieldOption.objects.filter(field=field):
                    self.fields[
                        'json_{0}_{1}'.format(
                            field.slug, fo.option.slug
                        )] = forms.CharField(required=False)
            else:
                self.fields[
                    'json_{0}'.format(field.slug)
                ] = forms.CharField(required=False)

    def clean(self):
        msg = _('The slug "%s" already exists on channel "%s" at site "%s"')
        data = self.cleaned_data
        if data.get('site') and data.get('channel') and data.get('slug'):
            qs = Container.objects.filter(
                site=data['site'], channel=data['channel'], slug=data['slug'])
            if self.instance.pk is not None:
                qs = qs.exclude(pk=self.instance.pk)
            if qs.exists():
                args = (data['slug'], data['channel'], data['site'])
                msg = msg % args
                self._errors["slug"] = self.error_class([msg])
        return data

    class Meta:
        model = Container



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url

from piston.resource import Resource

from opps.containers.api import ContainerHandler, ContainerBoxHandler


container = Resource(handler=ContainerHandler)
containerbox = Resource(handler=ContainerBoxHandler)

urlpatterns = patterns(
    '',
    url(r'^container/$', container, {'emitter_format': 'json'}),
    url(r'^containerbox/$', containerbox, {'emitter_format': 'json'}),
)



# coding: utf-8

"""
Text parsing, analysing and searching utilities
"""

import re
import htmlentitydefs


def unescape(text):
    """
    Removes HTML or XML character references and entities from a text string.

    @param text The HTML (or XML) source text.
    @return The plain text, as a Unicode string, if necessary.
    """

    if not text:
        return text

    def fixup(m):
        text = m.group(0)
        if text[:2] == "&#":
            # character reference
            try:
                if text[:3] == "&#x":
                    return unichr(int(text[3:-1], 16))
                else:
                    return unichr(int(text[2:-1]))
            except ValueError:
                pass
        else:
            # named entity
            try:
                text = unichr(htmlentitydefs.name2codepoint[text[1: -1]])
            except KeyError:
                pass
        return text  # leave as is
    return re.sub("&#?\w+;", fixup, text)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.test import TestCase
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site

from opps.containers.models import Container
from opps.channels.models import Channel
from opps.db import Db

from .models import Notification


class ContainerModelTest(TestCase):

    def setUp(self):
        User = get_user_model()
        self.user = User.objects.create(username=u'test', password='test')
        self.site = Site.objects.filter(name=u'example.com').get()
        self.channel = Channel.objects.create(name=u'Home', slug=u'home',
                                              description=u'home page',
                                              site=self.site, user=self.user)

        self.container = Container.objects.create(title=u'test',
                                                  user=self.user,
                                                  published=True,
                                                  site=self.site,
                                                  channel=self.channel)
        self.notification = Notification.objects.create(
            user=self.user,
            container=self.container,
            message='Test new notification',
            type='text'
        )

    def test_exist_notification(self):
        noti = Notification.objects.all()[0]
        self.assertTrue(self.notification)
        self.assertTrue(noti)
        self.assertEqual(noti, self.notification)
        self.assertEqual(noti.type, 'text')
        self.assertEqual(noti.id, self.notification.id)

    def test_pubsub_notification(self):
        noti = Notification.objects.all()[0]
        _db = Db(noti.container.get_absolute_url(),
                 noti.container.id)
        pubsub = _db.object().pubsub()
        pubsub.subscribe(_db.key)

        self.assertTrue(pubsub)

    def test_duplicate_notification(self):
        notification = Notification.objects.create(
            user=self.user,
            container=self.container,
            message='Test new notification',
            type='text'
        )
        get_notification = Notification.objects.all()
        self.assertTrue(notification)
        self.assertEqual(len(get_notification), 2)
        self.assertEqual(get_notification[0].channel_long_slug,
                         get_notification[1].channel_long_slug)
        self.assertEqual(get_notification[0].slug,
                         get_notification[1].slug)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django import forms
from django.utils.functional import lazy
from django.utils.translation import ugettext_lazy as _

from .models import QuerySet
from opps.core.forms import model_choices


class QuerySetAdminForm(forms.ModelForm):
    model = forms.ChoiceField(label=_(u'Model'),
                              choices=lazy(model_choices, tuple)())

    class Meta:
        model = QuerySet



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
import time
from django.conf import settings
from django.http import StreamingHttpResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt

from opps.views.generic.list import ListView
from opps.views.generic.json_views import JSONPResponse
from opps.db import Db

from .models import Notification


class AsyncServer(ListView):
    model = Notification

    def _db(self, obj):
        _db = Db(
            obj.container.get_absolute_url(),
            obj.container.id)
        pubsub = _db.object().pubsub()
        pubsub.subscribe(_db.key)
        return pubsub

    def _queue(self):
        try:
            obj = self.get_queryset()[0]
        except:
            obj = False

        if not obj:
            while True:
                msg = u"compatibility: true\n"
                msg += u"retry: 10000\n"
                msg += u"data: {0}\n\n".format(
                    json.dumps({"action": "error"}))
                yield msg
                time.sleep(10)
        else:
            while True:
                pubsub = self._db(obj)
                for m in pubsub.listen():
                    if m['type'] == 'message':
                        msg = u"compatibility: true\n"
                        msg += u"retry: 10000\n"
                        msg += u"data: {0}\n\n".format(m['data'])
                        yield msg
                ping = u"compatibility: true\n"
                ping += u"retry: 10000\n"
                ping += u"data: {0}\n\n".format(
                    json.dumps({"action": "ping"}))
                yield ping
                time.sleep(0.5)

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        response = StreamingHttpResponse(self._queue(),
                                         mimetype='text/event-stream')
        response['Cache-Control'] = 'no-cache'
        response['Software'] = 'opps-liveblogging'
        response['Access-Control-Allow-Origin'] = '*'
        response.flush()
        return response


class LongPullingServer(ListView, JSONPResponse):
    model = Notification

    def get_template_names(self):
        templates = []
        domain_folder = self.get_template_folder()

        if not self.long_slug:
            templates.append('{0}/none.json'.format(domain_folder))
            return templates

        list_name = 'list'

        if self.template_name_suffix:
            list_name = "{0}{1}".format(list_name, self.template_name_suffix)

        if self.channel:
            # Check layout, change via admin
            if self.channel.layout != u'default':
                list_name = self.channel.layout

            if self.channel.group and self.channel.parent:
                templates.append('{0}/{1}/{2}.json'.format(
                    domain_folder, self.channel.parent.long_slug, list_name))

                if self.request.GET.get('page') and\
                   self.__class__.__name__ not in\
                   settings.OPPS_PAGINATE_NOT_APP:
                    templates.append('{0}/{1}/{2}_paginated.json'.format(
                        domain_folder, self.channel.parent.long_slug,
                        list_name))

            if self.request.GET.get('page') and\
               self.__class__.__name__ not in settings.OPPS_PAGINATE_NOT_APP:
                templates.append('{0}/{1}/{2}_paginated.json'.format(
                    domain_folder, self.channel.long_slug, list_name))

            templates.append('{0}/{1}/{2}.json'.format(
                domain_folder, self.channel.long_slug, list_name))

            for t in self.channel.get_ancestors()[::-1]:
                templates.append('{0}/{1}/{2}.json'.format(
                    domain_folder, t.long_slug, list_name))
                if self.request.GET.get('page') and\
                   self.__class__.__name__ not in\
                   settings.OPPS_PAGINATE_NOT_APP:
                    templates.append('{0}/{1}/{2}_paginated.json'.format(
                        domain_folder, t.long_slug, list_name))

        if self.request.GET.get('page') and\
           self.__class__.__name__ not in settings.OPPS_PAGINATE_NOT_APP:
            templates.append('{0}/{1}_paginated.json'.format(domain_folder,
                                                             list_name))

        templates.append('{0}/{1}.json'.format(domain_folder, list_name))
        return templates

    def get_queryset(self):
        query = super(LongPullingServer, self).get_queryset()
        old_id = self.request.GET.get('old_id', 0)
        return query.filter(id__gte=old_id)._clone()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.contrib import admin
from .models import Notification


class AdminNotification(admin.ModelAdmin):
    raw_id_fields = ('container',)


admin.site.register(Notification, AdminNotification)



# coding: utf-8

import celery
from .models import Notification
from django.contrib.auth import get_user_model

User = get_user_model()


@celery.task
def notify(message, type='text',
           action='message', container=None, user=None):
    if not user:
        user = User.objects.all()[0]
    notification = Notification.objects.create(
        message=message,
        type=type,
        action=action,
        container=container,
        user=user
    )
    return notification



# coding: utf-8
from django.contrib.admin import SimpleListFilter
from django.utils.translation import ugettext_lazy as _
from django.contrib.sites.models import get_current_site
from django.db.models import Max, Count
from django.utils import timezone
from django.contrib.auth import get_user_model

from opps.channels.models import Channel
from opps.containers.models import Container

User = get_user_model()


class ChannelListFilter(SimpleListFilter):
    # Human-readable title which will be displayed in the
    # right admin sidebar just above the filter options.
    title = _(u'Channel')

    # Parameter for the filter that will be used in the URL query.
    parameter_name = 'channel'

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples. The first element in each
        tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        """

        qs = model_admin.queryset(request)
        qs = qs.order_by(
            'channel_long_slug'
        ).distinct().values('channel_long_slug')

        if qs:
            channels = set([(item['channel_long_slug'] or 'nochannel',
                             item['channel_long_slug'] or _(u'No channel'))
                           for item in qs])

            long_slug_list = sorted([i[0] for i in channels])
            items = []

            for channel in channels:
                items.append(channel)
                _value = channel[0]
                if self._get_descendant_count(_value, long_slug_list) > 0:
                    value = "{0}/*".format(_value)
                    human_readable = "{0}/*".format(_value)
                    items.append((value, human_readable))

            return sorted(items)

    def _get_descendant_count(self, item, channel_list):
        """
        Search item occurrences on channel_list
        """
        children = []
        item_set = set(item.split('/'))
        for channel in channel_list:
            splt = set(channel.split('/'))
            if item != channel and item_set.issubset(splt):
                children.append(channel)
        return len(children)

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.
        """
        value = self.value()
        if value == "nochannel":
            queryset = queryset.filter(channel_long_slug__isnull=True)
        elif value and "*" in value:
            site = get_current_site(request)
            long_slug = value.replace('/*', '')
            channel = Channel.objects.filter(site=site, long_slug=long_slug)[0]
            child_channels = channel.get_descendants(include_self=True)
            queryset = queryset.filter(channel__in=child_channels)
        elif value:
            queryset = queryset.filter(channel_long_slug=value)

        return queryset


class ChildClassListFilter(SimpleListFilter):
    # Human-readable title which will be displayed in the
    # right admin sidebar just above the filter options.
    title = _(u'Child class')

    # Parameter for the filter that will be used in the URL query.
    parameter_name = 'child_class'

    def lookups(self, request, model_admin):
        site = get_current_site(request)
        child_classes = [
            (i['child_class'], _(i['child_class'])) for i in
            Container.objects.values('child_class').filter(
                published=True,
                date_available__lte=timezone.now(),
                site=site).annotate(child=Count('child_class'),
                                    date=Max('date_available')
                                    ).order_by('-date')]
        return child_classes

    def queryset(self, request, queryset):
        child_class = self.value()
        if child_class:
            queryset = queryset.filter(child_class=child_class)
        return queryset


class HasQuerySet(SimpleListFilter):
    # Human-readable title which will be displayed in the
    # right admin sidebar just above the filter options.
    title = _(u'Has queryset')

    # Parameter for the filter that will be used in the URL query.
    parameter_name = 'hasqueryset'

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples. The first element in each
        tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        """
        return (
            ('no', _(u'No')),
            ('yes', _(u'Yes'))
        )

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.
        """
        if self.value() == "no":
            queryset = queryset.filter(queryset__isnull=True)
        elif self.value() == 'yes':
            queryset = queryset.filter(queryset__isnull=False)

        return queryset


class UserListFilter(SimpleListFilter):
    # Human-readable title which will be displayed in the
    # right admin sidebar just above the filter options.
    title = _(u'User')

    # Parameter for the filter that will be used in the URL query.
    parameter_name = u'user'

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples. The first element in each
        tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        """
        # filter only users with images
        qs = User.objects.filter(image__isnull=False).distinct()
        if qs:
            return set([(item.username,
                         u"{0} ({1})".format(item.get_full_name(), item.email))
                       for item in qs])

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.
        """
        if self.value() == u"nouser":
            queryset = queryset.filter(user__isnull=True)
        elif self.value():
            queryset = queryset.filter(user__username=self.value())

        return queryset



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _


FIELD_TYPE = (
    ('checkbox', _('Checkbox')),
    ('radio', _('Radio')),
    ('text', _('Text')),
    ('textarea', _('Textarea')),
)


class Field(models.Model):
    name = models.CharField(_('Name'), max_length=100)
    slug = models.SlugField(_('Slug'), max_length=255)
    application = models.CharField(_('Application'),
                                   max_length=255,
                                   db_index=True)
    type = models.CharField(_("Type"), max_length=15,
                            choices=FIELD_TYPE,
                            db_index=True)

    def __unicode__(self):
        return u"{0} - {1}".format(self.application, self.name)

    class Meta:
        verbose_name = _(u'Field')
        verbose_name_plural = _(u'Fields')


class Option(models.Model):
    field = models.ForeignKey('fields.Field')
    name = models.CharField(_('Name'), max_length=100)
    slug = models.SlugField(_('Slug'), max_length=140)
    value = models.CharField(_('Value'), max_length=255)

    def __unicode__(self):
        return u"{0} - {1}".format(self.field.slug, self.name)

    class Meta:
        verbose_name = _(u'Option')
        verbose_name_plural = _(u'Options')


class FieldOption(models.Model):
    field = models.ForeignKey('fields.Field')
    option = models.ForeignKey('fields.Option')
    order = models.PositiveIntegerField(_(u'Order'), default=0)

    def __unicode__(self):
        return u"{0} - {1}".format(self.field.slug, self.option.slug)

    class Meta:
        ordering = ['-order']
        verbose_name = _('Field option')
        verbose_name_plural = _('Field options')



# coding: utf-8

from django.db.models import get_models, get_app
from django.template import loader, TemplateDoesNotExist


def get_app_model(appname, suffix=""):
    app_label = appname.split('.')[-1]
    models = [model for model in get_models(get_app(app_label))
              if (model.__name__.endswith(suffix) or not suffix)
              and model._meta.app_label == app_label]
    return models and models[0]


def class_load(name):
    mod = __import__(name)
    components = name.split('.')
    for comp in components[1:]:
        mod = getattr(mod, comp)
    return mod


def get_template_path(path):
    try:
        template = loader.find_template(path)
        if template[1]:
            return template[1].name
        for template_loader in loader.template_source_loaders:
            try:
                source, origin = template_loader.load_template_source(path)
                return origin
            except TemplateDoesNotExist:
                pass
        raise TemplateDoesNotExist(path)
    except TemplateDoesNotExist:
        return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from opps.core.models import NotUserPublishable


class Logging(NotUserPublishable):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True, blank=True,
        verbose_name=_(u'User')
    )
    application = models.CharField(
        _(u"Application"),
        max_length=75,
        null=True, blank=True,
        db_index=True)
    action = models.CharField(
        _(u"Action"),
        max_length=50,
        null=True, blank=True,
        db_index=True)
    text = models.TextField(
        _(u"Text"),
        null=True, blank=True,
        db_index=True)

    def save(self, *args, **kwargs):
        self.published = True
        super(Logging, self).save(*args, **kwargs)

    class Meta:
        verbose_name = _(u'Logging')
        verbose_name_plural = _(u'Loggings')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django_coverage.coverage_runner import CoverageRunner


class Runner(CoverageRunner):
    def build_suite(self, *args, **kwargs):
        suite = super(Runner, self).build_suite(*args, **kwargs)
        tests = []
        for case in suite:
            pkg = case.__class__.__module__.split('.')[0]
            if pkg in ['opps']:
                tests.append(case)
        suite._tests = tests
        return suite



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.http import HttpResponse
from django.contrib.auth import authenticate
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger

from piston.handler import BaseHandler as Handler
from piston.emitters import JSONEmitter, Emitter

from opps.api.models import ApiKey


class UncachedEmitter(JSONEmitter):
    """ In websites running under varnish or another cache
    caching the api can mess the results and return the wrong data
    this emmitter injects No-Cache headers in response"""

    def render(self, request):
        content = super(UncachedEmitter, self).render(request)
        response = HttpResponse(content)
        response['Cache-Control'] = 'no-cache, no-store, must-revalidate'
        response['Content-Type'] = 'application/json; charset=utf-8'
        response['Pragma'] = 'no-cache'
        response['Expires'] = 0
        return response

Emitter.register('json', UncachedEmitter, 'application/json; charset=utf-8')


class BaseHandler(Handler):
    limit = 20
    limit_arg = 'paginate_limit'
    meta = {}

    def include_meta(self, d):
        obj = {'meta': self.meta, 'objects': d}
        return obj

    def paginate_queryset(self, queryset, request):
        limit = request.GET.get(self.limit_arg, self.meta.get(self.limit_arg))
        paginator = Paginator(queryset, limit or self.limit)

        self.meta['num_pages'] = paginator.num_pages
        self.meta['page_range'] = paginator.page_range
        self.meta['total_objects'] = paginator.count
        self.meta['per_page'] = paginator.per_page

        page = self.meta.get('page', request.GET.get('page', 1))

        try:
            results = paginator.page(page)
        except PageNotAnInteger:
            results = paginator.page(1)
        except EmptyPage:
            results = paginator.page(paginator.num_pages)

        self.meta['has_next'] = results.has_next()
        self.meta['has_previous'] = results.has_previous()
        self.meta['has_other_pages'] = results.has_other_pages()
        self.meta['end_index'] = results.end_index()
        self.meta['start_index'] = results.start_index()
        self.meta['page_number'] = results.number

        return results

    def read(self, request):
        base = self.model.objects

        if request.GET.items():
            items = request.GET.dict()
            self.meta[self.limit_arg] = items.pop(self.limit_arg, None)
            self.meta['page'] = items.pop('page', 1)
            qs = base.filter(**items)
        else:
            qs = base.all()

        self.meta['total_objects'] = qs.count()
        return qs

    def _limit(self, request):
        limit = request.GET.get(self.limit_arg, self.limit)
        return int(limit) * int(request.GET.get('page', 1))

    def _page(self, request):
        page = int(request.GET.get('page', 1))
        if page == 1:
            return 0
        limit = int(request.GET.get(self.limit_arg, self.limit))
        return limit * page - page

    def appendModel(Model, Filters):
        m = Model.objects.filter(**Filters)
        l = []
        for i in m:
            l.append(i.__dict__)
        return l


class ApiKeyAuthentication(object):
    def __init__(self, auth_func=authenticate, method=['GET']):
        self.auth_func = auth_func
        self.method = method

    def is_authenticated(self, request):
        if request.method == 'GET' and 'GET' in self.method:
            return True

        try:
            method = getattr(request, request.method)
        except:
            method = request.GET

        try:
            ApiKey.objects.get(
                user__username=method.get('api_username'),
                key=method.get('api_key'))
        except ApiKey.DoesNotExist:
            return False

        return True

    def challenge(self):
        resp = HttpResponse("Authorization Required")
        resp.status_code = 401
        return resp



# -*- coding: utf-8 -*-
from django.utils import timezone
from django.conf import settings
from django.contrib.sites.models import get_current_site
from django.core.cache import cache

from opps.containers.models import Container
from .models import Channel


def channel_context(request):
    """ Channel context processors
    """
    site = get_current_site(request)
    if settings.OPPS_MENU:
        opps_menu = cache.get('opps_menu')
        if not opps_menu:
            extra_lookups = {}
            if settings.OPPS_MENU_ONLY_WITH_PUBLISHED_CONTAINERS:
                extra_lookups = \
                    Container.objects.get_all_published_lookups("container__")

            opps_menu = [channel for channel in Channel.objects.filter(
                site=site,
                date_available__lte=timezone.now(),
                published=True,
                show_in_menu=True,
                **extra_lookups).distinct().order_by('order')]
            cache.set('opps_menu', opps_menu, settings.OPPS_CACHE_EXPIRE)
    else:
        opps_menu = []

    return {'opps_menu': opps_menu,
            'opps_channel_conf_all': settings.OPPS_CHANNEL_CONF,
            'site': site}



# -*- coding: utf-8 -*-
from django import template

register = template.Library()


@register.filter
def ofKey(value, arg):
    if value:
        return value.get(arg)
    return ""


@register.filter
def callMethod(obj, args):
    arg = args.split(',')
    methodName = arg[0]
    method = getattr(obj, methodName)
    return method(arg[1])


@register.assignment_tag
def try_values(*args):
    """Return the first valid value"""

    for arg in args:
        if arg:
            return arg

    return ""


@register.assignment_tag
def cache_obj(obj):
    """Cache given obj"""

    if not obj:
        return ""

    return obj



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django import forms
from django.utils.functional import lazy

from .models import Field
from opps.core.forms import model_choices


class FieldAdminForm(forms.ModelForm):
    application = forms.ChoiceField(choices=lazy(model_choices, tuple)())

    def __init__(self, *args, **kwargs):
        super(FieldAdminForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        if instance and instance.pk:
            self.fields['slug'].widget.attrs['readonly'] = True

    class Meta:
        model = Field



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.contrib import admin

from .models import Logging


class LoggingAdmin(admin.ModelAdmin):
    model = Logging
    raw_id_fields = ('user',)
    exclude = ('site_iid', 'site_domain', 'mirror_site')


admin.site.register(Logging, LoggingAdmin)
